<?php

 /**
  * @file
  *
  * Code for displaying the weather page.
  */

/**
 *
 * Implements hook_form().
 */
function weather_block_page_form() {

  $locations = array();

  $array = taxonomy_vocabulary_get_names();

  if (arg(0) == 'weather') {
    $values = taxonomy_get_tree($array['weather_locations']->vid);
  }
  if (arg(0) == 'world_weather') {
    $values = taxonomy_get_tree($array['weather_locations_foreign']->vid);
  }

  foreach ($values as $tid) {
    $locations[$tid->tid] = $tid->name;
  }

  if (arg(0) == 'weather') {
    if (isset($_SESSION['selected_weather_location'])) {
      if (taxonomy_term_load($_SESSION['selected_weather_location']) != NULL) {
        $location = $_SESSION['selected_weather_location'];
      }
      else {
        $location = variable_get('weather_location');
      }
    }
    else {
      $location = variable_get('weather_location');
    }
  }

  if (arg(0) == 'world_weather') {
    if (isset($_SESSION['selected_weather_location_world'])) {
      if (taxonomy_term_load($_SESSION['selected_weather_location_world']) != NULL) {
        $location = $_SESSION['selected_weather_location_world'];
      }
      else {
        $location = variable_get('weather_location_world');
      }
    }
    else {
      $location = variable_get('weather_location_world');
    }
  }

  $form['weather_location'] = array(
    '#type' => 'select',
    '#title' => t('Select city'),
    '#default_value' => $location,
    '#options' => $locations,
    '#required' => FALSE,
  );

  $form['weather_location_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Change location'),
  );

  return $form;
}

/**
 * Implements hook_form_submit().
 */
function weather_block_page_form_submit(&$form, $form_state) {

  if (arg(0) == 'weather') {
    $_SESSION['selected_weather_location'] = $form_state['values']['weather_location'];
  }
  if (arg(0) == 'world_weather') {
    $_SESSION['selected_weather_location_world'] = $form_state['values']['weather_location'];
  }
}

/**
 * Function for creating the page for a weather display for cities other than the default.
 */
function weather_block_page() {

  drupal_theme_rebuild();

  if (arg(0) == 'weather') {
    if (isset($_SESSION['selected_weather_location'])) {
      $location = taxonomy_term_load($_SESSION['selected_weather_location']);
    }
    else {
      $location = taxonomy_term_load(variable_get('weather_location'));
    }
  }

  if (arg(0) == 'world_weather') {
    if (isset($_SESSION['selected_weather_location_world'])) {
      $location = taxonomy_term_load($_SESSION['selected_weather_location_world']);
    }
    else {
      $location = taxonomy_term_load(variable_get('weather_location_world'));
    }
  }

  $services = variable_get('weather_services');

  foreach ($services as $key => $val) {
    if ($key == variable_get('weather_service')) {
      $key = 'field_' . $key;

      $location_id = $location->$key;
    }
  }

  $weather = new Weather($location_id['und'][0]['value'], variable_get('weather_service'));

  if (variable_get('weather_service') == 'weather_com') {
    $content = $weather->fiveDayForecast();
  }
  elseif (variable_get('weather_service') == 'yahoo_weather') {
    $data = $weather->otherCities();

    if (variable_get('weather_display_units') == 0) {
      $scale = t('&deg;C');
    }
    else {
      $scale = t('&deg;F');
    }

    $path = base_path() . drupal_get_path('module', 'weather_block');

    foreach ($data as $key => $val) {
      if (variable_get('weather_use_provider_weather_icons') == 1) {
        $icon = "http://l.yimg.com/a/i/us/we/52/" . $val['code'] . ".gif";
      }
      else {
        $icon = $path . "/weather-icons/" . $val['code'] . ".png";
      }

      $content[$key]['icon'] = check_plain($icon);

      if ($key == 0) {
        $content[$key]['current'] = check_plain($val['temp']) . " " . $scale;
        $content[$key]['day'] = t('Current weather');

        if (variable_get('weather_display_wind_yahoo_weather', 1) == 1) {
          $content[$key]['wind'] = "<div class='weather_wind'><div class='weather_wind_direction'>" . t('Wind');

          $imgpath = $path . "/weather-icons/wind/" . $val['wind']['direction'] . ".png";

          $content[$key]['wind'] .= " " . check_plain($weather->general->units->attributes()->speed) . "</div></div>";
        }

        if (variable_get('weather_display_humidity_yahoo_weather', 1) == 1) {
          $content[$key]['humidity'] = check_plain($val['humidity']) . "%";
        }
      }
      else {

        $content[$key]['low'] = check_plain($val['temp']['low']) . " " . $scale;
        $content[$key]['high'] = check_plain($val['temp']['high']) . " " . $scale;

        switch ($val['day']) {
          case 'Mon':
            $day = 'Monday';
          break;
          case 'Tue':
            $day = 'Tuesday';
          break;
          case 'Wed':
            $day = 'Wednesday';
          break;
          case 'Thu':
            $day = 'Thursday';
          break;
          case 'Fri':
            $day = 'Friday';
          break;
          case 'Sat':
            $day = 'Saturday';
          break;
          case 'Sun':
            $day = 'Sunday';
          break;
        }

        $content[$key]['day'] = t($day);
      }
    }
  }
  elseif (variable_get('weather_service') == 'google_weather') {
    $data = $weather->fourDayForecast();

    if (variable_get('weather_display_units') == 0) {
      $scale = t('&deg;C');
    }
    else {
      $scale = t('&deg;F');
    }

    foreach ($data as $key => $val) {
      switch ($val->day_of_week->attributes()->data) {
        case 'Mon':
          $day = 'Monday';
        break;
        case 'Tue':
          $day = 'Tuesday';
        break;
        case 'Wed':
          $day = 'Wednesday';
        break;
        case 'Thu':
          $day = 'Thursday';
        break;
        case 'Fri':
          $day = 'Friday';
        break;
        case 'Sat':
          $day = 'Saturday';
        break;
        case 'Sun':
          $day = 'Sunday';
        break;
      }

      $content[$key]['day'] = "<div class='forecast_day_wrapper'>" . t($day) . "</div>";

      if ($weather->xml->forecast_information->unit_system->attributes()->data == 'US' && variable_get('weather_display_units') == 0) {
        $deg_low = $weather->fahrenheitToCelsius($val->low->attributes()->data);
      }
      elseif ($weather->xml->forecast_information->unit_system->attributes()->data == 'SI' && variable_get('weather_display_units') == 1) {
        $deg_low = $weather->celsiusToFahrenheit($val->low->attributes()->data);
      }
      else {
        $deg_low = $val->low->attributes()->data;
      }

      $content[$key]['low'] = check_plain($deg_low) . " " . $scale;

      if ($weather->xml->forecast_information->unit_system->attributes()->data == 'US' && variable_get('weather_display_units') == 0) {
        $deg_high = $weather->fahrenheitToCelsius($val->high->attributes()->data);
      }
      elseif ($weather->xml->forecast_information->unit_system->attributes()->data == 'SI' && variable_get('weather_display_units') == 1) {
        $deg_high = $weather->celsiusToFahrenheit($val->high->attributes()->data);
      }
      else {
        $deg_high = $val->high->attributes()->data;
      }

      $content[$key]['high'] = check_plain($deg_high) . " " . $scale;

      $content[$key]['icon'] = "http://www.google.com" . check_plain($val->icon->attributes()->data);
    }
  }
  else {
  }

  $menu = $weather->menu();

  $form = drupal_get_form('weather_block_page_form');

  cache_clear_all();

  return theme('weather_block_local_weather', array('content' => $content, 'menu' => $menu, 'location' => $location, 'form' => $form, 'service' => variable_get('weather_service')));
}
